<?php

use \PayPal\Api\Amount;
use \PayPal\Api\Details;
use \PayPal\Api\Item;
use \PayPal\Api\ItemList;
use \PayPal\Api\Payer;
use \PayPal\Api\Payment;
use \PayPal\Api\RedirectUrls;
use \PayPal\Api\Transaction;

class Payment_model extends CI_Model
{
	public function __construct() {
		parent::__construct();
		// 1. Autoload the SDK Package. This will include all the files and classes to your autoloader
		$this->load->library('paypal');
	}

	public function create()
	{	
		// Payer:
		// A resource representing a Payer that funds a payment For paypal account payments, set payment method to 'paypal'.
		$payer = new Payer();
		$payer->setPaymentMethod("paypal");

		$items = array();
		foreach ($this->cart->contents() as $key => $item) {
			// check if item exists in cart
			if ($item['rowid'] == TRUE) {
				// configure cart items into paypal items
				$paypalItem = new Item();
				$paypalItem->setName($item['name'])
				    ->setCurrency($item['options']['currency'])
				    ->setQuantity($item['qty'])
				    ->setPrice($item['price']);
		    	array_push($items, $paypalItem);
			}
		}

		$itemList = new ItemList();
		$itemList->setItems($items);
		
		// Additional payment details:
		// Use this optional field to set additional payment information such as tax, shipping charges etc.
		$details = new Details();
		$details->setShipping(1.2)
		    ->setTax(1.3)
		    ->setSubtotal(17.50);
		
		// Amount:
		// Lets you specify a payment amount.
	    // You can also specify additional details such as shipping, tax.
		$amount = new Amount();
		$amount->setCurrency("USD")
		    ->setTotal(20)
		    ->setDetails($details);

		// Transaction:
		// A transaction defines the contract of a payment - 
	    // what is the payment for and who is fulfilling it.
		$transaction = new Transaction();
		$transaction->setAmount($amount)
		    ->setItemList($itemList)
		    ->setDescription("Payment description")
		    ->setInvoiceNumber(uniqid());

		// Redirect urls:
		// Set the urls that the buyer must be redirected to after payment approval/ cancellation.
		$redirectUrls = new RedirectUrls();
		$redirectUrls->setReturnUrl(base_url()."cart/checkout/success")
		    ->setCancelUrl(base_url()."cart/checkout/failed");
		
		// Payment:
		// A Payment Resource; create one using the above types and intent set to 'sale'
		$payment = new Payment();
		$payment->setIntent("sale")
		    ->setPayer($payer)
		    ->setRedirectUrls($redirectUrls)
		    ->setTransactions(array($transaction));

		// For Sample Purposes Only.
		$request = clone $payment;

		// Create Payment:
		// Create a payment by calling the 'create' method passing it a valid apiContext.
		// (See bootstrap.php for more on ApiContext) The return object contains the state
		// and the url to which the buyer must be redirected to for payment approval
		try {
			$apiContext = new \PayPal\Rest\ApiContext(
			    new \PayPal\Auth\OAuthTokenCredential(
			        'ATIwihZv8uYLwDWJI8yUGz9KX-aViTtrhtoy3YMe8dhh4UIZZiKVZFMn4hai9n_T-z1oqsernr6YJlPx',     // ClientID
			        'ELiWUuZHsQI1XTEVbQaBuzLOauiNxWfT3tThcbN0kMO8iT9snAZ2jfRUYcnCCIuVhV9VoxmeV0AYkrmu'      // ClientSecret
			    )
			);
			$apiContext->setConfig(
			  array(
			  	'mode' => 'live',
			    'log.LogEnabled' => true,
			    'log.FileName' => 'PayPal.log',
			    'log.LogLevel' => 'FINE'
			  )
			);
			
		    $payment->create($apiContext);
		} catch (Exception $ex) {
			var_dump("failed");
			echo $ex->xdebug_message;
		    // var_dump("Created Payment Using PayPal. Please visit the URL to Approve.", "Payment", null, $ex);
		    exit(1);
		}

		// Get redirect url:
		// The API response provides the url that you must redirect the buyer to. Retrieve the url from the $payment->getApprovalLink() method
		$approvalUrl = $payment->getApprovalLink();

		var_dump("Created Payment Using PayPal. Please visit the URL to Approve.", "Payment", "<a href='$approvalUrl' >$approvalUrl</a>", $request, $payment);

		return $payment;
	}
}