<?php
class Fetch extends CI_Model
{

	protected $database;

	public function __construct()
	{
		$this->load->database();
	}

	public function store()
	{
		$storeData = $this->db->get('store')->result();
		$data 	   = $storeData[0];
		$location  = $data->home;

		// ensure that there are geocodes in the database
		if ($storeData[0]->latitude == "" || $storeData[0]->longitude == "")
		{
			$latitude  = $storeData[0]->latitude;
			$longitude = $storeData[0]->longitude;
		}
		// otherwise download geocodes from google api
		else
		{
			$geocode = @file_get_contents('http://maps.google.com/maps/api/geocode/json?address='.str_replace(" ", "+", $location).'&sensor=false');
			
			// ensure that the api returns content
			if ($geocode === false)
			{
				$latitude  = false;
				$longitude = false;
			}
			else
			{
				$output    = json_decode($geocode);
				$latitude  = $output->results[0]->geometry->location->lat;
				$longitude = $output->results[0]->geometry->location->lng;
			}
		}
		return array(
			"latitude"   => $latitude,
			"longitude"  => $longitude,
			"location"   => $location,
			"data" 		 => $data
		);
	}

	public function slides()
	{
		// Ensure that the product table exists
		if ($this->db->table_exists('slideshow'))
		{
			$this->db->order_by('id', 'RANDOM')->limit(4);
			$data = $this->db->get('slideshow')->result();

			// Check if the query returns any records
			if (is_array($data) && count($data) == 0 || empty($data)) {
				return FALSE;
			} else {
				return $data;
			}
		} else {
			return FALSE;
		}
	}

	public function banners()
	{
		// Ensure that the product table exists
		if ($this->db->table_exists('banner'))
		{
			$this->db->order_by('id', 'RANDOM')->limit(12);
			$data = $this->db->get('banner')->result();

			// Check if the query returns any records
			if (is_array($data) && count($data) == 0 || empty($data)) {
				return FALSE;
			} else {
				return $data;
			}
		} else {
			return FALSE;
		}
	}

	public function products($number = false)
	{
		// Ensure that the product table exists
		if ($this->db->table_exists('product')) {
			if ($number !== false) {
				$this->db->limit($number);
			}
			$this->db->order_by('id', 'RANDOM');

			$items = $this->db->get('product')->result();
			// Check if the query returns any records
			if (empty($items)) {
				return false;
			} else {
				$results = array();
				foreach ($items as $key => $item) {
					$array = array('product_id' => $item->product_id);
					$query = $this->db->get_where('product_image', $array)->result();
					$image = $query[0]->image_url;


					array_push($results, array(
						'id'	=> $item->product_id,
						'name'	=> $item->name,
						'desc'	=> $item->description,
						'price'	=> $item->price,
						'image' => $image
					));
				}
				return $results;
			}
		} else {
			return false;
		}
	}

	public function categories()
	{
		$result = array();
			   $this->db->order_by('product_category_id', 'RANDOM');
		$cat = $this->db->get('product_category')->result();
		
		foreach ($cat as $key => $category) {
			$array = array('category' => $category->category);
					 $this->db->order_by('id', 'RANDOM');
			$items = $this->db->get_where('product', $array)->result();
			if (!empty($items)) {
				foreach ($items as $key => $item) {
					$array = array('product_id' => $item->product_id);
					$query = $this->db->get_where('product_image', $array)->result();
					$data['image'] = $query[0]->image_url;
				}
				$data['category'] = $category->category;
			}
			if (isset($data) AND !in_array($data, $result)) {
				array_push($result, $data);
			}
		}
		return $result;
	}
}