<?php
class Update_model extends CI_Model
{
	protected $database;

	public function __construct()
	{
		$this->load->database();
	}

	/*
	|--------------------------------------------------------------------------
	| Update product description to the appropriate tables
	|--------------------------------------------------------------------------
	*/
	public function update_product_details()
	{
		$product_id 	= $this->input->post('product_id');
		$query_array 	= array('product_id' => $product_id);


		// update the product table
		$product['name'] 		 = $this->input->post('name');
		$product['description']  = $this->input->post('description');
		$product['price'] 		 = $this->input->post('price');
		$product['category'] 	 = $this->input->post('category');
		$product['sub_category'] = $this->input->post('sub_category');
		$product['type'] 		 = $this->input->post('type');
		$product['slug'] 		 = str_replace(" ", "_", $product['name']);

		$this->db->where('product_id', $product_id);
		$this->db->update('product', $product);


		// delete current color records
		$this->db->delete('product_color', $query_array);

		// add new color records
		$colors = array_unique($this->input->post('color'));
		foreach ($colors as $key => $color)
		{
			$productColorData = array(
				'product_id' => $product_id,
				'color' 	 => $color
			);
			$this->db->insert('product_color', $productColorData);
		}

		// delete current size records
		$this->db->delete('product_size', $query_array);

		// add new size records
		$sizes = array_unique($this->input->post('size'));
		foreach ($sizes as $key => $size)
		{
			$productSizeData = array(
				'product_id' => $product_id,
				'size' 		 => $size
			);
			$this->db->insert('product_size', $productSizeData);
		}

		// delete current quantity records
		$this->db->delete('product_quantity', $query_array);

		// add new quantity records
		foreach ($this->input->post('quantity') as $key => $quantity) {
			
			$colors = $this->input->post('color');
			$sizes  = $this->input->post('size');
			
			$array['product_id'] 	= $product_id;
			$array['product_color']	= $colors[$key];
			$array['product_size']	= $sizes[$key];
			$array['quantity']		= $quantity;

			$this->db->insert('product_quantity', $array);
		}

		$attributes = $this->input->post("attributes");
		
		foreach ($attributes as $key => $attribute) {

			$attributeArray  = explode("_", $attribute);

			$value = str_replace("%20", "_", $attributeArray[0]);
			$field = str_replace(" ", "_", $value);
			$field = str_replace("-", "_", $field);

			if (isset($attributeArray[1])) {
				// new attributes set
				$description 	 = $attributeArray[1];

				if (!$this->db->field_exists($field, 'product_description')) {
					// Create the column
					$q = "ALTER TABLE product_description ADD ".$field." VARCHAR(220);";
					$this->db->query($q);
				}
				$this->db->where('product_id', $product_id);
				// set nulls as undefined
				if ($description == "") {
					$description = "undefined";
				}
				$this->db->where('product_id', $product_id);
				$this->db->update('product_description', array($field => $description));
			}

		}
	}

	/*
	|--------------------------------------------------------------------------
	| Update product images
	|--------------------------------------------------------------------------
	*/
	public function update_product_thumbnail($ID = FALSE)
	{
		
		// get the image url and product_id
		$query_array = array('product_image_id' => $ID);
		$image_query = $this->db->get_where('product_image', $query_array)->result();
		$image_url	 = $image_query[0]->image_url;
		$product_id  = $image_query[0]->product_id;

		// get current thumbnail url
		$query_array = array('product_id' => $product_id);
		$thumb_query = $this->db->get_where('product', $query_array)->result();
		$thumb_path  = "./".$thumb_query[0]->thumb;


		// define the image paths
		$image_path	 = "./".$image_url;
		$file_name 	 = str_replace("./assets/images/products/", "", $image_path);
		$destination = "./assets/images/products/thumb/".$file_name;

		// copy the image to thumb folder
		copy($image_path, $destination);

		// load the CI image library with these configurations
		// it resizes the thumbnail to smaller dimensions
		$config['image_library'] 	= 'gd2';
		$config['source_image']		= $destination;
		$config['file_name']		= $destination;
		$config['create_thumb'] 	= FALSE;
		$config['maintain_ratio'] 	= TRUE;
		$config['width']	 		= 250;
		$config['height']			= 250;

		$this->load->library('image_lib');
		$this->image_lib->initialize($config);
		$this->image_lib->resize();

		// Update the image thumbnail path in the product table
		$new_image_path = str_replace("./", "", $destination);
		$update_data 	= array('thumb' => $new_image_path);
		$this->db->where('product_id', $product_id);
		$this->db->update('product', $update_data);

		// delete current thumbnail
		unlink($thumb_path);
	}

	/*
	|--------------------------------------------------------------------------
	| Update product images
	|--------------------------------------------------------------------------
	*/
	public function update_product_images()
	{
		$product_id 		= $this->input->post("product_id");
		$upload_path 		= "./assets/images/products/";
		$upload_thumb_path 	= "./assets/images/products/thumb/";
		
		// load and initialize the CI upload library
		$this->load->library('upload');
		$this->upload->initialize(array(
			"upload_path" 	=> $upload_path,
			"overwrite" 	=> TRUE,
			"encrypt_name" 	=> TRUE,
			"remove_spaces" => FALSE,
			"allowed_types" => "gif|jpg|png",
			"max_size" 		=> 2000,
			"xss_clean" 	=> FALSE,
		));

		if ($this->upload->do_multi_upload("files"))
		{
			$images = $this->upload->get_multi_upload_data();

			// resize each uploaded image
			for ($i=0; $i < count($images); $i++)
			{
				$image_url 	= $upload_path.$images[$i]['file_name'];
				$path 		= str_replace("./", "", $image_url);

				// load the CI image library with these configurations
				$config['image_library'] 	= 'gd2';
				$config['source_image']		= $image_url;
				$config['file_name']		= $image_url;
				$config['create_thumb'] 	= FALSE;
				$config['maintain_ratio'] 	= FALSE;
				$config['width']	 		= 370;
				$config['height']			= 370;

				$this->load->library('image_lib');
				$this->image_lib->initialize($config);
				$this->image_lib->resize();

				// Insert the images' paths into the product_image table
				$data = array(
					'product_id' => $product_id,
					'image_url'  => $path
				);
				$this->db->insert('product_image', $data);

				return $this->db->get_where('product_image', $data)->result();
			}
		}
	}
}