<?php
class Product_model extends CI_Model
{
	public function __construct()
	{
		$this->load->database();
	}

	public function get($number = false)
	{
		// Ensure that the product table exists
		if ($this->db->table_exists('product')) {
			if ($number !== false) {
					$this->db->limit($number);
			}
					$this->db->order_by('id', 'RANDOM');
			$data = $this->db->get('product')->result();

			// Check if the query returns any records
			if (empty($data)) {
				return false;
			} else {
				return $data;
			}
		} else {
			return false;
		}
	}

	public function getDetails($ID, $option)
	{
		// get product information
		$info_query  = $this->db->get_where('product', array('product_id' => $ID))->result();
		$productInfo = $info_query[0];

		if ($option == "editable") {
			// get product images and their IDs
						   $this->db->select('product_image_id');
						   $this->db->select('image_url');
			$image_query = $this->db->get_where('product_image', array('product_id' => $ID))->result();
			$images 	 = array();
			
			foreach ($image_query as $key => $value) {
				array_push($images, $value);
			}
			
			// get product quantity
			$quantity_query = $this->db->get_where('product_quantity', array('product_id' => $ID))->result();
			$quantity 		= array();

			foreach ($quantity_query as $key => $value) {
				array_push($quantity, $value);
			}
		} else {

			// get product images
						   $this->db->select('image_url');
			$image_query = $this->db->get_where('product_image', array('product_id' => $ID))->result();
			$images 	 = array();

			foreach ($image_query as $key => $value) {
				array_push($images, $value->image_url);
			}

			// get product colors
						   $this->db->select('color');
			$color_query = $this->db->get_where('product_color', array('product_id' => $ID))->result();
			$colors 	 = array();

			foreach ($color_query as $key => $value) {
				array_push($colors, $value->color);
			}

			// get product sizes
						  $this->db->select('size');
			$size_query = $this->db->get_where('product_size', array('product_id' => $ID))->result();
			$sizes 		= array();

			foreach ($size_query as $key => $value) {
				array_push($sizes, $value->size);
			}

			// get product quantity
							  $this->db->select('quantity');
			$quantity_query = $this->db->get_where('product_quantity', array('product_id' => $ID))->result();
			$quantity 		= array();

			foreach ($quantity_query as $key => $value) {
				array_push($quantity, $value->quantity);
			}
		}


		// get product category ID
		$category_array = array('category' => $productInfo->category);
		$category_query = $this->db->get_where('product_category', $category_array)->result();
		$categoryID 	= $category_query[0]->product_category_id;

		// get product sub-category ID
		$sub_category_array = array(
			'product_category_id' 	=> $categoryID,
			'sub_category'			=> $productInfo->sub_category
		);
		$sub_category_query = $this->db->get_where('product_sub_category', $sub_category_array)->result();
		$sub_categoryID 	= $sub_category_query[0]->product_sub_category_id;

		// get product type ID
		$type_array = array(
			'product_category_id' 		=> $categoryID,
			'product_sub_category_id' 	=> $sub_categoryID,
			'type'						=> $productInfo->type
		);
		$type_query = $this->db->get_where('product_type', $type_array)->result();
		$typeID 	= $type_query[0]->product_type_id;

		// get product attributes of a specific type
					  $this->db->select('attribute');
		$attributes = $this->db->get_where('product_attribute', array('product_type_id' => $typeID))->result();


		foreach ($attributes as $key => $value) {

			// convert attribute string value to a table field value
			$attribute = str_replace(" ", "_", $value->attribute);
			$attribute = str_replace("-", "_", $attribute);

			// check that the field is set
			if ($this->db->field_exists($attribute, 'product_description')) {
				$this->db->select($attribute);
			} else {
				// the user created attribute but not description values
			}
		}
		$description_query 	= $this->db->get_where('product_description', array('product_id' => $ID))->result_array();
		$descriptions 		= $description_query[0];

		$productData = array(
			'ID' 			=> $productInfo->product_id,
			'name' 			=> $productInfo->name,
			'describe' 		=> $productInfo->description,
			'thumb' 		=> $productInfo->thumb,
			'price' 		=> $productInfo->price,
			'category' 		=> $productInfo->category,
			'sub_category' 	=> $productInfo->sub_category,
			'type' 			=> $productInfo->type,
			'images' 		=> $images,
			'colors' 		=> $colors,
			'sizes' 		=> $sizes,
			'quantity' 		=> $quantity,
			'attributes' 	=> $attributes,
			'description' 	=> $descriptions
		);
		return $productData;
	}
	
	/*
	|-----------------------------------------------------------------------------------------------
	| Fetch products of the same type
	|-----------------------------------------------------------------------------------------------
	*/
	public function get_related_products($type)
	{
		// get all products
						  $this->db->order_by('id', 'RANDOM');
						  $this->db->limit(8);
		$productQuery   = $this->db->get_where('product', array('type' => $type));
		$productResult  = $productQuery->result();

		return $productResult;
	}

	/*
	|-----------------------------------------------------------------------------------------------
	| Fetch products of a specific type
	|-----------------------------------------------------------------------------------------------
	| Only a category that contains products in it will be returned
	*/
	public function get_type($category, $sub_category, $type)
	{
		$queryArray = array(
			"category" 		=> $category,
			"sub_category" 	=> $sub_category,
			"type" 			=> $type
		);

		// Get the products category specified
		$productQuery   = $this->db->get_where('product', $queryArray);
		$productResult 	= $productQuery->result();

		if (empty($productResult)) {
			// This category has no products
		} else {
			return $productResult;
		}
	}

	/*
	|-----------------------------------------------------------------------------------------------
	| Fetch products of a given category
	|-----------------------------------------------------------------------------------------------
	*/
	public function get_by_category($category)
	{
		$array 	  = array('category'=>$category);
		$products = $this->db->get_where('product', $array)->result();

		return $products;
	}
	
	/*
	|-----------------------------------------------------------------------------------------------
	| Fetch product's quantity based on its size and quantity
	|-----------------------------------------------------------------------------------------------
	*/
	public function getProductQuantity($ID, $size, $color)
	{
		$ID    = str_replace("%20", " ", $ID);
		$size  = str_replace("%20", " ", $size);
		$color = str_replace("%20", " ", $color);
		$array = array(
			'product_id' 	=> $ID,
			'product_size'	=> $size,
			'product_color'	=> $color,
		);
		$query 		= $this->db->get_where('product_quantity', $array);
		$result 	= $query->result();
		$quantity 	= $result[0]->quantity;

		return $quantity;
	}
}