<?php
class Search_model extends CI_Model {

	public function __construct()
	{
		$this->load->database();
	}


	// The two functions convert sizes and colors from the arrays into a string
	public function get_sizes($size_array)
	{
		$string = '';
		foreach ($size_array as $key => $array) {
			$string = $string." ".$array['size'];
		}
		return $string;
	}
	public function get_colors($size_array)
	{
		$string = '';
		foreach ($size_array as $key => $array) {
			$string = $string." ".$array['color'];
		}
		return $string;
	}

	public function query($user_query, $category = false)
	{
		$products = modules::run('productDB/get/products');
		
		if ($category == false) {
			$filtered = false;
		} else {
			$filtered = modules::run('productDB/get/full_category', $category);
		}
		$productData = array();

		if (!empty($products)) {
			// Get relevant product data: description, colors and sizes,
			// add create full_description.
			// this is what the search algorithm will consume
			foreach ($products as $product) {
				$array = array('product_id' => $product->product_id);
				// Get product colors and sizes
				$colors = $this->db->get_where('product_color', $array)->result_array();
				$sizes  = $this->db->get_where('product_size', $array)->result_array();

				// Get product attribute descriptions
				$description = "";
				$arrayDesc   = $this->db->get_where('product_description', $array)->result();

				if (isset($arrayDesc[0])) {
					foreach ($arrayDesc[0] as $key => $value) {
						if ($value !== NULL OR !is_numeric($value)) {
							$description = $description." ".$value;
						}
					}
				}

				// Compile all info into an array
				// Only full_description will be consume by the search algorithm
				$compiled = array();

				$compiled["product_id"] 	  = $product->product_id;
				$compiled["name"] 			  = $product->name;
				$compiled["category"] 		  = $product->category;
				$compiled["sub_category"] 	  = $product->sub_category;
				$compiled["type"] 			  = $product->type;
				$compiled["price"] 			  = $product->price;
				$compiled["thumb"] 			  = $product->thumb;
				$compiled["full_description"] = $product->name." ".
												$product->category." ".
												$product->description." ".
												$product->sub_category." ".
												$product->type." ".$description." ".
												$this->get_sizes($sizes)." ".
												$this->get_colors($colors);
				// Determine if the target matches the search query
				$compiled["matching_terms"]   = 0;
				// Rate the acurracy of the search result
				$compiled["score"]   		  = 0;

				array_push($productData, $compiled);
			}
		}

		// Load Library porterstemmmer search algorithm
		$this->load->library('porterstemmer');

		// Remove some characters and words.($rmWords and $rmSymbols)
		$rmWords 	= array("the", "and", "a", "to", "of", "in", "i", "is",
		"that", "it", "on", "you", "this", "for", "but",
		"with", "are", "have", "be", "at", "or", "as",
		"was", "so", "if", "out", "not");
		$rmSymbols 	= array('\'s', '.', ',', '\'', '!', '?', '"');

		$searchTerms 	= explode(" ", str_replace($rmSymbols, "", $user_query));
		$stemmerTerms 	= array();
		foreach ($searchTerms as $searchTerm) {
			$stem = PorterStemmer::Stem($searchTerm);
			if (!in_array($stem, $rmWords)) {
				$stemmerTerms[] = $stem;
			}
		}
		
		$searchResults = array();
		
		foreach ($productData as $pdt) {
	    	// Remove symbols 
			$words 		= str_replace($rmSymbols, " ", $pdt['full_description']);
			$words 		= explode(" ", $words);
			$stem_words = array();
			foreach($words as $word) {
				$stem = PorterStemmer::Stem($word);
				if(!in_array($stem, $rmWords)) {
					$stem_words[] = $stem;
					if(in_array($stem, $stemmerTerms)) {
						$pdt['matching_terms']++;
					}
				}
			}
			if ($pdt['matching_terms'] > 0) {
			    // Generate a score
				$pdt['score'] = ( $pdt['matching_terms'] / count( $stem_words ) ) * 100;
		    	// record this result
				$searchResults[] = $pdt;
			}
		}
    	// Sort the results
		usort($searchResults, function($a,$b)
		{
			return $b['score']-$a['score'];
		});

		// Group the results by category, sub_category and type
		$this->load->library('underscore');
		$groupType 	 = underscore::groupBy($searchResults, 'type');
		$groupCat 	 = underscore::groupBy($searchResults, 'category');
		$groupSubCat = underscore::groupBy($searchResults, 'sub_category');

		// Fetch the product categories, sub-categories and types from the db
		$productCat  = modules::run('productDB/get/menu');

		$results['product'] 				= $searchResults;
		$results['filtered'] 				= $filtered;
		$results['type'] 					= $groupType;
		$results['category'] 				= $groupCat;
		$results['sub_category'] 			= $groupSubCat;
		$results['product_categories'] 		= $productCat;

		return $results;
	}
}